@@ -1,7 +1,7 @@
 /*
 * libslack - http://libslack.org/
 *
-* Copyright (C) 1999-2002, 2004, 2010, 2020-2021 raf <raf@raf.org>
+* Copyright (C) 1999-2002, 2004, 2010, 2020-2023 raf <raf@raf.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <https://www.gnu.org/licenses/>.
 *
-* 20210220 raf <raf@raf.org>
+* 20230313 raf <raf@raf.org>
 */
 
 #ifndef LIBSLACK_ERR_H
@@ -40,10 +40,10 @@
 #undef check
 
 #ifdef NDEBUG
-#define debug(args)
-#define vdebug(args)
-#define debugsys(args)
-#define vdebugsys(args)
+#define debug(args) ;
+#define vdebug(args) ;
+#define debugsys(args) ;
+#define vdebugsys(args) ;
 #define check(cond, mesg) (void_cast(0))
 #else
 #define debug(args) debugf args;
